@@ -425,7 +425,8 @@ class _HashedKey(list):
         return '%s(%s)' % (self.__class__.__name__, list.__repr__(self))
 
 
-def make_cache_key(args, kwargs, typed=False, kwarg_mark=_KWARG_MARK,
+def make_cache_key(args, kwargs, typed=False,
+                   kwarg_mark=_KWARG_MARK,
                    fasttypes=frozenset([int, str, frozenset, type(None)])):
     """Make a generic key from a function's positional and keyword
     arguments, suitable for use in caches. Arguments within *args* and
@@ -444,6 +445,9 @@ def make_cache_key(args, kwargs, typed=False, kwarg_mark=_KWARG_MARK,
 
     .. _hashable: https://docs.python.org/2/glossary.html#term-hashable
     """
+
+    # key = [func_name] if func_name else []
+    # key.extend(args)
     key = list(args)
     if kwargs:
         sorted_items = sorted(kwargs.items())
@@ -465,7 +469,7 @@ class CachedFunction(object):
     """This type is used by :func:`cached`, below. Instances of this
     class are used to wrap functions in caching logic.
     """
-    def __init__(self, func, cache, typed=False):
+    def __init__(self, func, cache, scoped=True, typed=False, key=None):
         self.func = func
         if callable(cache):
             self.get_cache = cache
@@ -478,11 +482,13 @@ class CachedFunction(object):
             def _get_cache():
                 return cache
             self.get_cache = _get_cache
+        self.scoped = scoped
         self.typed = typed
+        self.key_func = key or make_cache_key
 
     def __call__(self, *args, **kwargs):
         cache = self.get_cache()
-        key = make_cache_key(args, kwargs, typed=self.typed)
+        key = self.key_func(args, kwargs, typed=self.typed)
         try:
             ret = cache[key]
         except KeyError:
@@ -491,8 +497,9 @@ class CachedFunction(object):
 
     def __repr__(self):
         cn = self.__class__.__name__
-        if self.typed:
-            return "%s(func=%r, typed=%r)" % (cn, self.func, self.typed)
+        if self.typed or not self.scoped:
+            return ("%s(func=%r, scoped=%r, typed=%r)"
+                    % (cn, self.func, self.scoped, self.typed))
         return "%s(func=%r)" % (cn, self.func)
 
 
@@ -500,7 +507,7 @@ class CachedMethod(object):
     """Similar to :class:`CachedFunction`, this type is used by
     :func:`cachedmethod` to wrap methods in caching logic.
     """
-    def __init__(self, func, cache, typed=False, selfish=True):
+    def __init__(self, func, cache, scoped=True, typed=False, key=None):
         self.func = func
         if isinstance(cache, basestring):
             self.get_cache = attrgetter(cache)
@@ -510,30 +517,30 @@ class CachedMethod(object):
                   and callable(getattr(cache, '__setitem__', None))):
             raise TypeError('expected cache to be an attribute name,'
                             ' dict-like object, or callable returning'
-                            ' a dict-like object, not %r'
-                            % cache)
+                            ' a dict-like object, not %r' % cache)
         else:
             def _get_cache(obj):
                 return cache
             self.get_cache = _get_cache
+        self.scoped = scoped
         self.typed = typed
-        self.selfish = selfish
+        self.key_func = key or make_cache_key
         self.bound_to = None
 
     def __get__(self, obj, objtype=None):
         if obj is None:
             return self
         cls = self.__class__
-        ret = cls(self.func, self.get_cache,
-                  typed=self.typed, selfish=self.selfish)
+        ret = cls(self.func, self.get_cache, typed=self.typed,
+                  scoped=self.scoped, key=self.key_func)
         ret.bound_to = obj
         return ret
 
     def __call__(self, *args, **kwargs):
         obj = args[0] if self.bound_to is None else self.bound_to
         cache = self.get_cache(obj)
-        key_args = (self.bound_to,) + args if self.selfish else args
-        key = make_cache_key(key_args, kwargs, typed=self.typed)
+        key_args = (self.bound_to, self.func) + args if self.scoped else args
+        key = self.key_func(key_args, kwargs, typed=self.typed)
         try:
             ret = cache[key]
         except KeyError:
@@ -544,14 +551,14 @@ class CachedMethod(object):
 
     def __repr__(self):
         cn = self.__class__.__name__
-        args = (cn, self.func, self.typed, self.selfish)
+        args = (cn, self.func, self.scoped, self.typed)
         if self.bound_to is not None:
             args += (self.bound_to,)
-            return ('<%s func=%r typed=%r selfish=%r bound_to=%r>' % args)
-        return ("%s(func=%r, typed=%r, selfish=%r)" % args)
+            return ('<%s func=%r scoped=%r typed=%r bound_to=%r>' % args)
+        return ("%s(func=%r, scoped=%r, typed=%r)" % args)
 
 
-def cached(cache, typed=False):
+def cached(cache, scoped=True, typed=False, key=None):
     """Cache any function with the cache object of your choosing. Note
     that the function wrapped should take only `hashable`_ arguments.
 
@@ -561,6 +568,12 @@ def cached(cache, typed=False):
             :class:`LRI` are good choices, but a plain :class:`dict`
             can work in some cases, as well. This argument can also be
             a callable which accepts no arguments and returns a mapping.
+        scoped (bool): Whether the function itself is part of the
+            cache key.  ``True`` by default, different functions will
+            not read one another's cache entries, but can evict one
+            another's results. ``False`` can be useful for certain
+            shared cache use cases. More advanced behavior can be
+            produced through the *key* argument.
         typed (bool): Whether to factor argument types into the cache
             check. Default ``False``, setting to ``True`` causes the
             cache keys for ``3`` and ``3.0`` to be considered unequal.
@@ -579,11 +592,11 @@ def cached(cache, typed=False):
 
     """
     def cached_func_decorator(func):
-        return CachedFunction(func, cache, typed=typed)
+        return CachedFunction(func, cache, scoped=scoped, typed=typed, key=key)
     return cached_func_decorator
 
 
-def cachedmethod(cache, typed=False, selfish=True):
+def cachedmethod(cache, scoped=True, typed=False, key=None):
     """Similar to :func:`cached`, ``cachedmethod`` is used to cache
     methods based on their arguments, using any :class:`dict`-like
     *cache* object.
@@ -592,12 +605,18 @@ def cachedmethod(cache, typed=False, selfish=True):
         cache (str/Mapping/callable): Can be the name of an attribute
             on the instance, any Mapping/:class:`dict`-like object, or
             a callable which returns a Mapping.
+        scoped (bool): Whether the method itself and the object it is
+            bound to are part of the cache keys. ``True`` by default,
+            different methods will not read one another's cache
+            results. ``False`` can be useful for certain shared cache
+            use cases. More advanced behavior can be produced through
+            the *key* arguments.
         typed (bool): Whether to factor argument types into the cache
             check. Default ``False``, setting to ``True`` causes the
             cache keys for ``3`` and ``3.0`` to be considered unequal.
-        selfish (bool): Whether an instance's ``self`` argument should
-            be considered in the cache key. Use this to share cache
-            objects across instances.
+        key (callable): A callable with a signature that matches
+            :func:`make_cache_key` that returns a tuple of hashable
+            values to be used as the key in the cache.
 
     >>> class Lowerer(object):
     ...     def __init__(self):
@@ -612,9 +631,10 @@ def cachedmethod(cache, typed=False, selfish=True):
     'wow who could guess caching could be so neat'
     >>> len(lowerer.cache)
     1
+
     """
     def cached_method_decorator(func):
-        return CachedMethod(func, cache, typed=typed, selfish=selfish)
+        return CachedMethod(func, cache, scoped=scoped, typed=typed, key=key)
     return cached_method_decorator
 
 
